@@ -2,17 +2,17 @@
 '''
 Common utilities
 '''
-import io
-import re
-import os
-import time
-import sys
 import errno
-import types
-import struct
-import socket
+import io
 import logging
+import os
+import re
+import socket
+import struct
+import sys
 import threading
+import time
+import types
 
 log = logging.getLogger(__name__)
 
@@ -24,65 +24,68 @@ try:
     reduce = reduce
     file = file
 
-    class PermissionError(Exception):
-        pass
-
 except NameError:
     #
     # Python3 section
     #
     basestring = (str, bytes)
     from functools import reduce
+
     reduce = reduce
     file = io.BytesIO
-    PermissionError = PermissionError
 
 AF_MPLS = 28
 AF_PIPE = 255  # Right now AF_MAX == 40
-DEFAULT_RCVBUF = 32768
+DEFAULT_RCVBUF = 65536
 _uuid32 = 0  # (singleton) the last uuid32 value saved to avoid collisions
 _uuid32_lock = threading.Lock()
 
-size_suffixes = {'b': 1,
-                 'k': 1024,
-                 'kb': 1024,
-                 'm': 1024 * 1024,
-                 'mb': 1024 * 1024,
-                 'g': 1024 * 1024 * 1024,
-                 'gb': 1024 * 1024 * 1024,
-                 'kbit': 1024 / 8,
-                 'mbit': 1024 * 1024 / 8,
-                 'gbit': 1024 * 1024 * 1024 / 8}
+size_suffixes = {
+    'b': 1,
+    'k': 1024,
+    'kb': 1024,
+    'm': 1024 * 1024,
+    'mb': 1024 * 1024,
+    'g': 1024 * 1024 * 1024,
+    'gb': 1024 * 1024 * 1024,
+    'kbit': 1024 / 8,
+    'mbit': 1024 * 1024 / 8,
+    'gbit': 1024 * 1024 * 1024 / 8,
+}
 
 
-time_suffixes = {'s': 1,
-                 'sec': 1,
-                 'secs': 1,
-                 'ms': 1000,
-                 'msec': 1000,
-                 'msecs': 1000,
-                 'us': 1000000,
-                 'usec': 1000000,
-                 'usecs': 1000000}
+time_suffixes = {
+    's': 1,
+    'sec': 1,
+    'secs': 1,
+    'ms': 1000,
+    'msec': 1000,
+    'msecs': 1000,
+    'us': 1000000,
+    'usec': 1000000,
+    'usecs': 1000000,
+}
 
-rate_suffixes = {'bit': 1,
-                 'Kibit': 1024,
-                 'kbit': 1000,
-                 'mibit': 1024 * 1024,
-                 'mbit': 1000000,
-                 'gibit': 1024 * 1024 * 1024,
-                 'gbit': 1000000000,
-                 'tibit': 1024 * 1024 * 1024 * 1024,
-                 'tbit': 1000000000000,
-                 'Bps': 8,
-                 'KiBps': 8 * 1024,
-                 'KBps': 8000,
-                 'MiBps': 8 * 1024 * 1024,
-                 'MBps': 8000000,
-                 'GiBps': 8 * 1024 * 1024 * 1024,
-                 'GBps': 8000000000,
-                 'TiBps': 8 * 1024 * 1024 * 1024 * 1024,
-                 'TBps': 8000000000000}
+rate_suffixes = {
+    'bit': 1,
+    'Kibit': 1024,
+    'kbit': 1000,
+    'mibit': 1024 * 1024,
+    'mbit': 1000000,
+    'gibit': 1024 * 1024 * 1024,
+    'gbit': 1000000000,
+    'tibit': 1024 * 1024 * 1024 * 1024,
+    'tbit': 1000000000000,
+    'Bps': 8,
+    'KiBps': 8 * 1024,
+    'KBps': 8000,
+    'MiBps': 8 * 1024 * 1024,
+    'MBps': 8000000,
+    'GiBps': 8 * 1024 * 1024 * 1024,
+    'GBps': 8000000000,
+    'TiBps': 8 * 1024 * 1024 * 1024 * 1024,
+    'TBps': 8000000000000,
+}
 
 
 ##
@@ -92,6 +95,7 @@ class View(object):
     '''
     A read-only view of a dictionary object.
     '''
+
     def __init__(self, src=None, path=None, constraint=lambda k, v: True):
         self.src = src if src is not None else {}
         if path is not None:
@@ -145,7 +149,6 @@ class View(object):
 
 
 class Namespace(object):
-
     def __init__(self, parent, override=None):
         self.parent = parent
         self.override = override or {}
@@ -192,11 +195,11 @@ class Dotkeys(dict):
     But it simplifies live for old-school admins, who works with good
     old "lo", "eth0", and like that naming schemes.
     '''
+
     __var_name = re.compile('^[a-zA-Z_]+[a-zA-Z_0-9]*$')
 
     def __dir__(self):
-        return [i for i in self if
-                type(i) == str and self.__var_name.match(i)]
+        return [i for i in self if type(i) == str and self.__var_name.match(i)]
 
     def __getattribute__(self, key, *argv):
         try:
@@ -205,9 +208,11 @@ class Dotkeys(dict):
             if key == '__deepcopy__':
                 raise e
             elif key[:4] == 'set_':
+
                 def set_value(value):
                     self[key[4:]] = value
                     return self
+
                 return set_value
             elif key in self:
                 return self[key]
@@ -255,16 +260,17 @@ def map_namespace(prefix, ns, normalize=None):
         - True — cut the prefix and `lower()` the rest
         - lambda x: … — apply the function to every name
     '''
-    nmap = {None: lambda x: x,
-            True: lambda x: x[len(prefix):].lower()}
+    nmap = {None: lambda x: x, True: lambda x: x[len(prefix) :].lower()}
 
     if not isinstance(normalize, types.FunctionType):
         normalize = nmap[normalize]
 
-    by_name = dict([(normalize(i), ns[i]) for i in ns.keys()
-                    if i.startswith(prefix)])
-    by_value = dict([(ns[i], normalize(i)) for i in ns.keys()
-                     if i.startswith(prefix)])
+    by_name = dict(
+        [(normalize(i), ns[i]) for i in ns.keys() if i.startswith(prefix)]
+    )
+    by_value = dict(
+        [(ns[i], normalize(i)) for i in ns.keys() if i.startswith(prefix)]
+    )
     return (by_name, by_value)
 
 
@@ -273,12 +279,12 @@ def getbroadcast(addr, mask, family=socket.AF_INET):
     i = socket.inet_pton(family, addr)
     if family == socket.AF_INET:
         i = struct.unpack('>I', i)[0]
-        a = 0xffffffff
+        a = 0xFFFFFFFF
         length = 32
     elif family == socket.AF_INET6:
         i = struct.unpack('>QQ', i)
         i = i[0] << 64 | i[1]
-        a = 0xffffffffffffffffffffffffffffffff
+        a = 0xFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
         length = 128
     else:
         raise NotImplementedError('family not supported')
@@ -301,34 +307,33 @@ def dqn2int(mask, family=socket.AF_INET):
     ret = 0
     binary = socket.inet_pton(family, mask)
     for offset in range(len(binary) // 4):
-        ret += bin(struct.unpack('I', binary[offset * 4:
-                                             offset * 4 + 4])[0]).count('1')
+        ret += bin(
+            struct.unpack('I', binary[offset * 4 : offset * 4 + 4])[0]
+        ).count('1')
     return ret
 
 
+def get_address_family(address):
+    if address.find(':') > -1:
+        return socket.AF_INET6
+    else:
+        return socket.AF_INET
+
+
 def hexdump(payload, length=0):
     '''
     Represent byte string as hex -- for debug purposes
     '''
-    if sys.version[0] == '3':
-        return ':'.join('{0:02x}'.format(c)
-                        for c in payload[:length] or payload)
-    else:
-        return ':'.join('{0:02x}'.format(ord(c))
-                        for c in payload[:length] or payload)
+    return ':'.join('{0:02x}'.format(c) for c in payload[:length] or payload)
 
 
 def hexload(data):
-    ret = ''.join(chr(int(x, 16)) for x in data.split(':'))
-    if sys.version[0] == '3':
-        return bytes(ret, 'ascii')
-    else:
-        return bytes(ret)
+    return bytes(bytearray((int(x, 16) for x in data.split(':'))))
 
 
 def load_dump(f, meta=None):
     '''
-    Load a packet dump from an open file-like object.
+    Load a packet dump from an open file-like object or a string.
 
     Supported dump formats:
 
@@ -345,35 +350,54 @@ def load_dump(f, meta=None):
     '''
     data = ''
     code = None
-    for a in f.readlines():
+    meta_data = None
+    meta_label = None
+    if isinstance(f, str):
+        io_obj = io.StringIO()
+        io_obj.write(f)
+        io_obj.seek(0)
+    else:
+        io_obj = f
+
+    for a in io_obj.readlines():
         if code is not None:
             code += a
             continue
 
+        if meta_data is not None:
+            meta_data += a
+            continue
+
         offset = 0
         length = len(a)
         while offset < length:
             if a[offset] in (' ', '\t', '\n'):
                 offset += 1
             elif a[offset] == '#':
-                if a[offset:offset + 2] == '#!':
-                    # read and save the code block;
-                    # do not parse it here
+                if a[offset : offset + 2] == '#!':
+                    # read the code block until EOF
                     code = ''
+                elif a[offset : offset + 2] == '#:':
+                    # read data block until EOF
+                    meta_label = a.split(':')[1].strip()
+                    meta_data = ''
                 break
             elif a[offset] == '.':
                 return data
             elif a[offset] == '\\':
                 # strace hex format
-                data += chr(int(a[offset + 2:offset + 4], 16))
+                data += chr(int(a[offset + 2 : offset + 4], 16))
                 offset += 4
             else:
                 # pyroute2 hex format
-                data += chr(int(a[offset:offset + 2], 16))
+                data += chr(int(a[offset : offset + 2], 16))
                 offset += 3
 
     if isinstance(meta, dict):
-        meta['code'] = code
+        if code is not None:
+            meta['code'] = code
+        if meta_data is not None:
+            meta[meta_label] = meta_data
 
     if sys.version[0] == '3':
         return bytes(data, 'iso8859-1')
@@ -385,13 +409,12 @@ class AddrPool(object):
     '''
     Address pool
     '''
-    cell = 0xffffffffffffffff
 
-    def __init__(self,
-                 minaddr=0xf,
-                 maxaddr=0xffffff,
-                 reverse=False,
-                 release=False):
+    cell = 0xFFFFFFFFFFFFFFFF
+
+    def __init__(
+        self, minaddr=0xF, maxaddr=0xFFFFFF, reverse=False, release=False
+    ):
         self.cell_size = 0  # in bits
         mx = self.cell
         self.reverse = reverse
@@ -433,7 +456,7 @@ class AddrPool(object):
                             self.addr_map[base] ^= 1 << bit
                             break
                         bit += 1
-                    ret = (base * self.cell_size + bit)
+                    ret = base * self.cell_size + bit
 
                     if self.reverse:
                         ret = self.maxaddr - ret
@@ -458,6 +481,28 @@ class AddrPool(object):
             else:
                 raise KeyError('no free address available')
 
+    def alloc_multi(self, count):
+        with self.lock:
+            addresses = []
+            raised = False
+            try:
+                for _ in range(count):
+                    addr = self.alloc()
+                    try:
+                        addresses.append(addr)
+                    except:
+                        # In case of a MemoryError during appending,
+                        # the finally block would not free the address.
+                        self.free(addr)
+                return addresses
+            except:
+                raised = True
+                raise
+            finally:
+                if raised:
+                    for addr in addresses:
+                        self.free(addr)
+
     def locate(self, addr):
         if self.reverse:
             addr = self.maxaddr - addr
@@ -486,8 +531,7 @@ class AddrPool(object):
     def free(self, addr, ban=0):
         with self.lock:
             if ban != 0:
-                self.ban.append({'addr': addr,
-                                 'counter': ban})
+                self.ban.append({'addr': addr, 'counter': ban})
             else:
                 base, bit, is_allocated = self.locate(addr)
                 if len(self.addr_map) <= base:
@@ -510,11 +554,11 @@ def _fnv1_python2(data):
 
     See: http://www.isthe.com/chongo/tech/comp/fnv/index.html
     '''
-    hval = 0x811c9dc5
+    hval = 0x811C9DC5
     for i in range(len(data)):
         hval *= 0x01000193
         hval ^= struct.unpack('B', data[i])[0]
-    return hval & 0xffffffff
+    return hval & 0xFFFFFFFF
 
 
 def _fnv1_python3(data):
@@ -529,11 +573,11 @@ def _fnv1_python3(data):
 
     See: http://www.isthe.com/chongo/tech/comp/fnv/index.html
     '''
-    hval = 0x811c9dc5
+    hval = 0x811C9DC5
     for i in range(len(data)):
         hval *= 0x01000193
         hval ^= data[i]
-    return hval & 0xffffffff
+    return hval & 0xFFFFFFFF
 
 
 if sys.version[0] == '3':
@@ -557,9 +601,9 @@ def uuid32():
     with _uuid32_lock:
         candidate = _uuid32
         while candidate == _uuid32:
-            candidate = fnv1(struct.pack('QQ',
-                                         int(time.time() * 1000000),
-                                         os.getpid()))
+            candidate = fnv1(
+                struct.pack('QQ', int(time.time() * 1000000), os.getpid())
+            )
         _uuid32 = candidate
         return candidate
 
@@ -578,6 +622,7 @@ def map_exception(match, subst):
     '''
     Decorator to map exception types
     '''
+
     def wrapper(f):
         def decorated(*argv, **kwarg):
             try:
@@ -586,7 +631,9 @@ def map_exception(match, subst):
                 if match(e):
                     raise subst(e)
                 raise
+
         return decorated
+
     return wrapper
 
 
@@ -594,10 +641,10 @@ def map_enoent(f):
     '''
     Shortcut to map OSError(2) -> OSError(95)
     '''
-    return map_exception(lambda x: (isinstance(x, OSError) and
-                                    x.errno == errno.ENOENT),
-                         lambda x: OSError(errno.EOPNOTSUPP,
-                                           'Operation not supported'))(f)
+    return map_exception(
+        lambda x: (isinstance(x, OSError) and x.errno == errno.ENOENT),
+        lambda x: OSError(errno.EOPNOTSUPP, 'Operation not supported'),
+    )(f)
 
 
 def metaclass(mc):
@@ -613,13 +660,12 @@ def metaclass(mc):
             if k not in skip:
                 nvars[k] = v
         return mc(cls.__name__, cls.__bases__, nvars)
+
     return wrapped
 
 
 def failed_class(message):
-
     class FailedClass(object):
-
         def __init__(self, *argv, **kwarg):
             ret = RuntimeError(message)
             ret.feature_supported = False
