@@ -74,27 +74,30 @@ MPLS routes
 See here: :ref:`mpls`
 
 '''
-import time
-import uuid
 import json
 import struct
-from socket import AF_INET
-from socket import AF_INET6
-from socket import inet_pton
-from functools import partial
+import time
+import uuid
 from collections import OrderedDict
-from pyroute2.ndb.objects import RTNL_Object
-from pyroute2.ndb.report import Record
-from pyroute2.ndb.auth_manager import check_auth
-from pyroute2.common import basestring
-from pyroute2.common import AF_MPLS
-from pyroute2.netlink.rtnl.rtmsg import rtmsg
-from pyroute2.netlink.rtnl.rtmsg import nh
-from pyroute2.netlink.rtnl.rtmsg import LWTUNNEL_ENCAP_MPLS
+from functools import partial
+from socket import AF_INET, inet_pton
+
+from pyroute2.common import AF_MPLS, basestring
+from pyroute2.netlink.rtnl.rtmsg import LWTUNNEL_ENCAP_MPLS, nh, rtmsg
+from pyroute2.requests.common import MPLSTarget
+from pyroute2.requests.route import RouteFieldFilter
+
+from ..auth_manager import check_auth
+from ..objects import RTNL_Object
+from ..report import Record
 
 _dump_rt = ['main.f_%s' % x[0] for x in rtmsg.sql_schema()][:-2]
 _dump_nh = ['nh.f_%s' % x[0] for x in nh.sql_schema()][:-2]
 
+F_RTA_MULTIPATH = 1
+F_RTA_ENCAP = 2
+F_RTA_METRICS = 4
+
 
 def get_route_id(schema, target, event):
     keys = ['f_target = %s' % schema.plch]
@@ -132,26 +135,30 @@ def load_rtmsg(schema, target, event):
     #   - table 254 (main)
     #   - proto 2 (kernel)
     #   - scope 253 (link)
-    elif (event.get_attr('RTA_TABLE') == 254) and \
-            (event['proto'] == 2) and \
-            (event['scope'] == 253) and \
-            (event['family'] == AF_INET):
+    elif (
+        (event.get_attr('RTA_TABLE') == 254)
+        and (event['proto'] == 2)
+        and (event['scope'] == 253)
+        and (event['family'] == AF_INET)
+    ):
         evt = event['header']['type']
         #
         # set f_gc_mark = timestamp for "del" events
         # and clean it for "new" events
         #
         try:
-            rtmsg_gc_mark(schema, target, event,
-                          int(time.time()) if (evt % 2) else None)
+            rtmsg_gc_mark(
+                schema, target, event, int(time.time()) if (evt % 2) else None
+            )
         except Exception as e:
-            schema.log.error('gc_mark event: %s' % (event, ))
-            schema.log.error('gc_mark: %s' % (e, ))
+            schema.log.error('gc_mark event: %s' % (event,))
+            schema.log.error('gc_mark: %s' % (e,))
 
     #
     # only for RTM_NEWROUTE events
     #
     if not event['header']['type'] % 2:
+        event['deps'] = 0
         #
         # RTA_MULTIPATH
         #
@@ -163,11 +170,15 @@ def load_rtmsg(schema, target, event):
             #
             # load multipath
             for idx in range(len(mp)):
-                mp[idx]['header'] = {}          # for load_netlink()
+                mp[idx]['header'] = {}  # for load_netlink()
                 mp[idx]['route_id'] = route_id  # set route_id on NH
-                mp[idx]['nh_id'] = idx          # add NH number
-                post.append(partial(schema.load_netlink, 'nh', target,
-                                    mp[idx], 'routes'))
+                mp[idx]['nh_id'] = idx  # add NH number
+                post.append(
+                    partial(
+                        schema.load_netlink, 'nh', target, mp[idx], 'routes'
+                    )
+                )
+                event['deps'] |= F_RTA_MULTIPATH
         #
         # RTA_ENCAP
         #
@@ -178,8 +189,12 @@ def load_rtmsg(schema, target, event):
             #
             encap['header'] = {}
             encap['route_id'] = route_id
-            post.append(partial(schema.load_netlink, 'enc_mpls', target,
-                                encap, 'routes'))
+            post.append(
+                partial(
+                    schema.load_netlink, 'enc_mpls', target, encap, 'routes'
+                )
+            )
+            event['deps'] |= F_RTA_ENCAP
         #
         # RTA_METRICS
         #
@@ -191,8 +206,12 @@ def load_rtmsg(schema, target, event):
             #
             metrics['header'] = {}
             metrics['route_id'] = route_id
-            post.append(partial(schema.load_netlink, 'metrics', target,
-                                metrics, 'routes'))
+            post.append(
+                partial(
+                    schema.load_netlink, 'metrics', target, metrics, 'routes'
+                )
+            )
+            event['deps'] |= F_RTA_METRICS
     #
     if route_id is not None:
         event['route_id'] = route_id
@@ -211,28 +230,27 @@ def rtmsg_gc_mark(schema, target, event, gc_mark=None):
     #
     # select all routes for that OIF where f_gc_mark is not null
     #
-    key_fields = ','.join(['f_%s' % x for x
-                           in schema.indices['routes']])
-    key_query = ' AND '.join(['f_%s = %s' % (x, schema.plch) for x
-                              in schema.indices['routes']])
-    routes = (schema
-              .execute('''
+    key_fields = ','.join(['f_%s' % x for x in schema.indices['routes']])
+    key_query = ' AND '.join(
+        ['f_%s = %s' % (x, schema.plch) for x in schema.indices['routes']]
+    )
+    routes = schema.execute(
+        '''
                        SELECT %s,f_RTA_GATEWAY FROM routes WHERE
                        f_target = %s AND f_RTA_OIF = %s AND
                        f_RTA_GATEWAY IS NOT NULL %s AND
                        f_family = 2
-                       ''' % (key_fields,
-                              schema.plch,
-                              schema.plch,
-                              gc_clause),
-                       (target, event.get_attr('RTA_OIF')))
-              .fetchmany())
+                       '''
+        % (key_fields, schema.plch, schema.plch, gc_clause),
+        (target, event.get_attr('RTA_OIF')),
+    ).fetchmany()
     #
     # get the route's RTA_DST and calculate the network
     #
     addr = event.get_attr('RTA_DST')
-    net = struct.unpack('>I', inet_pton(AF_INET, addr))[0] &\
-        (0xffffffff << (32 - event['dst_len']))
+    net = struct.unpack('>I', inet_pton(AF_INET, addr))[0] & (
+        0xFFFFFFFF << (32 - event['dst_len'])
+    )
     #
     # now iterate all the routes from the query above and
     # mark those with matching RTA_GATEWAY
@@ -243,74 +261,94 @@ def rtmsg_gc_mark(schema, target, event, gc_mark=None):
         try:
             gwnet = struct.unpack('>I', inet_pton(AF_INET, gw))[0] & net
             if gwnet == net:
-                (schema
-                 .execute('UPDATE routes SET f_gc_mark = %s '
-                          'WHERE f_target = %s AND %s'
-                          % (schema.plch, schema.plch, key_query),
-                          (gc_mark, target) + route[:-1]))
+                (
+                    schema.execute(
+                        'UPDATE routes SET f_gc_mark = %s '
+                        'WHERE f_target = %s AND %s'
+                        % (schema.plch, schema.plch, key_query),
+                        (gc_mark, target) + route[:-1],
+                    )
+                )
         except Exception as e:
-            schema.log.error('gc_mark event: %s' % (event, ))
+            schema.log.error('gc_mark event: %s' % (event,))
             schema.log.error('gc_mark: %s : %s' % (e, route))
 
 
-rt_schema = (rtmsg
-             .sql_schema()
-             .push('route_id', 'TEXT UNIQUE')
-             .push('gc_mark', 'INTEGER')
-             .unique_index('family',
-                           'dst_len',
-                           'tos',
-                           'scope',
-                           'RTA_DST',
-                           'RTA_OIF',
-                           'RTA_PRIORITY',
-                           'RTA_TABLE',
-                           'RTA_VIA',
-                           'RTA_NEWDST')
-             .foreign_key('interfaces',
-                          ('f_target', 'f_tflags', 'f_RTA_OIF'),
-                          ('f_target', 'f_tflags', 'f_index'))
-             .foreign_key('interfaces',
-                          ('f_target', 'f_tflags', 'f_RTA_IIF'),
-                          ('f_target', 'f_tflags', 'f_index')))
+rt_schema = (
+    rtmsg.sql_schema()
+    .push('route_id', 'TEXT UNIQUE')
+    .push('gc_mark', 'INTEGER')
+    .push('deps', 'INTEGER')
+    .unique_index(
+        'family',
+        'dst_len',
+        'tos',
+        'scope',
+        'type',
+        'RTA_DST',
+        'RTA_OIF',
+        'RTA_PRIORITY',
+        'RTA_TABLE',
+        'RTA_VIA',
+        'RTA_NEWDST',
+    )
+    .foreign_key(
+        'interfaces',
+        ('f_target', 'f_tflags', 'f_RTA_OIF'),
+        ('f_target', 'f_tflags', 'f_index'),
+    )
+    .foreign_key(
+        'interfaces',
+        ('f_target', 'f_tflags', 'f_RTA_IIF'),
+        ('f_target', 'f_tflags', 'f_index'),
+    )
+)
 
-nh_schema = (nh
-             .sql_schema()
-             .push('route_id', 'TEXT')
-             .push('nh_id', 'INTEGER')
-             .unique_index('route_id', 'nh_id')
-             .foreign_key('routes', ('f_route_id', ), ('f_route_id', ))
-             .foreign_key('interfaces',
-                          ('f_target', 'f_tflags', 'f_oif'),
-                          ('f_target', 'f_tflags', 'f_index')))
+nh_schema = (
+    nh.sql_schema()
+    .push('route_id', 'TEXT')
+    .push('nh_id', 'INTEGER')
+    .unique_index('route_id', 'nh_id')
+    .foreign_key('routes', ('f_route_id',), ('f_route_id',))
+    .foreign_key(
+        'interfaces',
+        ('f_target', 'f_tflags', 'f_oif'),
+        ('f_target', 'f_tflags', 'f_index'),
+    )
+)
 
-metrics_schema = (rtmsg
-                  .metrics
-                  .sql_schema()
-                  .push('route_id', 'TEXT')
-                  .unique_index('route_id')
-                  .foreign_key('routes', ('f_route_id', ), ('f_route_id', )))
+metrics_schema = (
+    rtmsg.metrics.sql_schema()
+    .push('route_id', 'TEXT')
+    .unique_index('route_id')
+    .foreign_key('routes', ('f_route_id',), ('f_route_id',))
+)
 
-mpls_enc_schema = (rtmsg
-                   .mpls_encap_info
-                   .sql_schema()
-                   .push('route_id', 'TEXT')
-                   .unique_index('route_id')
-                   .foreign_key('routes', ('f_route_id', ), ('f_route_id', )))
+mpls_enc_schema = (
+    rtmsg.mpls_encap_info.sql_schema()
+    .push('route_id', 'TEXT')
+    .unique_index('route_id')
+    .foreign_key('routes', ('f_route_id',), ('f_route_id',))
+)
 
-init = {'specs': [['routes', rt_schema],
-                  ['nh', nh_schema],
-                  ['metrics', metrics_schema],
-                  ['enc_mpls', mpls_enc_schema]],
-        'classes': [['routes', rtmsg],
-                    ['nh', nh],
-                    ['metrics', rtmsg.metrics],
-                    ['enc_mpls', rtmsg.mpls_encap_info]],
-        'event_map': {rtmsg: [load_rtmsg]}}
+init = {
+    'specs': [
+        ['routes', rt_schema],
+        ['nh', nh_schema],
+        ['metrics', metrics_schema],
+        ['enc_mpls', mpls_enc_schema],
+    ],
+    'classes': [
+        ['routes', rtmsg],
+        ['nh', nh],
+        ['metrics', rtmsg.metrics],
+        ['enc_mpls', rtmsg.mpls_encap_info],
+    ],
+    'event_map': {rtmsg: [load_rtmsg]},
+}
 
 
 class Via(OrderedDict):
-
     def __init__(self, prime=None):
         super(OrderedDict, self).__init__()
         if prime is None:
@@ -321,37 +359,11 @@ class Via(OrderedDict):
         self['addr'] = prime.get('addr', '0.0.0.0')
 
     def __eq__(self, right):
-        return isinstance(right, (dict, Target)) and \
-            self['family'] == right.get('family', AF_INET) and \
-            self['addr'] == right.get('addr', '0.0.0.0')
-
-    def __repr__(self):
-        return repr(dict(self))
-
-
-class Target(OrderedDict):
-
-    def __init__(self, prime=None):
-        super(OrderedDict, self).__init__()
-        if prime is None:
-            prime = {}
-        elif isinstance(prime, int):
-            prime = {'label': prime}
-        elif isinstance(prime, dict):
-            pass
-        else:
-            raise TypeError()
-        self['label'] = prime.get('label', 16)
-        self['tc'] = prime.get('tc', 0)
-        self['bos'] = prime.get('bos', 1)
-        self['ttl'] = prime.get('ttl', 0)
-
-    def __eq__(self, right):
-        return isinstance(right, (dict, Target)) and \
-            self['label'] == right.get('label', 16) and \
-            self['tc'] == right.get('tc', 0) and \
-            self['bos'] == right.get('bos', 1) and \
-            self['ttl'] == right.get('ttl', 0)
+        return (
+            isinstance(right, (dict))
+            and self['family'] == right.get('family', AF_INET)
+            and self['addr'] == right.get('addr', '0.0.0.0')
+        )
 
     def __repr__(self):
         return repr(dict(self))
@@ -363,9 +375,23 @@ class Route(RTNL_Object):
     msg_class = rtmsg
     hidden_fields = ['route_id']
     api = 'route'
+    field_filter = RouteFieldFilter
 
     _replace_on_key_change = True
 
+    @classmethod
+    def _count(cls, view):
+        if view.chain:
+            return view.ndb.schema.fetchone(
+                'SELECT count(*) FROM %s WHERE f_RTA_OIF = %s'
+                % (view.table, view.ndb.schema.plch),
+                [view.chain['index']],
+            )
+        else:
+            return view.ndb.schema.fetchone(
+                'SELECT count(*) FROM %s' % view.table
+            )
+
     @classmethod
     def _dump_where(cls, view):
         if view.chain:
@@ -374,7 +400,10 @@ class Route(RTNL_Object):
                     WHERE
                         main.f_target = %s AND
                         main.f_RTA_OIF = %s
-                    ''' % (plch, plch)
+                    ''' % (
+                plch,
+                plch,
+            )
             values = [view.chain['target'], view.chain['index']]
         else:
             where = ''
@@ -384,37 +413,44 @@ class Route(RTNL_Object):
     @classmethod
     def summary(cls, view):
         req = '''
-              WITH nr AS
+              WITH main AS
                   (SELECT
-                      main.f_target, main.f_tflags, main.f_RTA_TABLE,
-                      main.f_RTA_DST, main.f_dst_len,
-                      CASE WHEN nh.f_oif > main.f_RTA_OIF
+                      nr.f_target, nr.f_tflags, nr.f_RTA_TABLE,
+                      nr.f_RTA_DST, nr.f_dst_len,
+                      CASE WHEN nh.f_oif > nr.f_RTA_OIF
                           THEN nh.f_oif
-                          ELSE main.f_RTA_OIF
+                          ELSE nr.f_RTA_OIF
                       END AS f_RTA_OIF,
                       CASE WHEN nh.f_RTA_GATEWAY IS NOT NULL
                           THEN nh.f_RTA_GATEWAY
-                          ELSE main.f_RTA_GATEWAY
+                          ELSE nr.f_RTA_GATEWAY
                       END AS f_RTA_GATEWAY
                    FROM
-                       routes AS main
+                       routes AS nr
                    LEFT JOIN nh
                    ON
-                       main.f_route_id = nh.f_route_id AND
-                       main.f_target = nh.f_target)
+                       nr.f_route_id = nh.f_route_id AND
+                       nr.f_target = nh.f_target)
               SELECT
-                  nr.f_target, nr.f_tflags, nr.f_RTA_TABLE,
-                  intf.f_IFLA_IFNAME, nr.f_RTA_DST, nr.f_dst_len,
-                  nr.f_RTA_GATEWAY
+                  main.f_target, main.f_tflags, main.f_RTA_TABLE,
+                  intf.f_IFLA_IFNAME, main.f_RTA_DST, main.f_dst_len,
+                  main.f_RTA_GATEWAY
               FROM
-                  nr
+                  main
               INNER JOIN interfaces AS intf
               ON
-                  nr.f_rta_oif = intf.f_index AND
-                  nr.f_target = intf.f_target
+                  main.f_rta_oif = intf.f_index AND
+                  main.f_target = intf.f_target
               '''
-        yield ('target', 'tflags', 'table', 'ifname',
-               'dst', 'dst_len', 'gateway')
+        yield (
+            'target',
+            'tflags',
+            'table',
+            'ifname',
+            'dst',
+            'dst_len',
+            'gateway',
+        )
         where, values = cls._dump_where(view)
         for record in view.ndb.schema.fetch(req + where, values):
             yield record
@@ -427,66 +463,78 @@ class Route(RTNL_Object):
               LEFT JOIN nh AS nh
               ON main.f_route_id = nh.f_route_id
                   AND main.f_target = nh.f_target
-              ''' % ','.join(['%s' % x for x in
-                              _dump_rt + _dump_nh + ['main.f_route_id']])
-        header = (['target', 'tflags'] +
-                  [rtmsg.nla2name(x[7:]) for x in _dump_rt] +
-                  ['nh_%s' % nh.nla2name(x[5:]) for x in _dump_nh] +
-                  ['metrics', 'encap'])
+              ''' % ','.join(
+            ['%s' % x for x in _dump_rt + _dump_nh + ['main.f_route_id']]
+        )
+        header = (
+            ['target', 'tflags']
+            + [rtmsg.nla2name(x[7:]) for x in _dump_rt]
+            + ['nh_%s' % nh.nla2name(x[5:]) for x in _dump_nh]
+            + ['metrics', 'encap']
+        )
         yield header
         plch = view.ndb.schema.plch
         where, values = cls._dump_where(view)
         for record in view.ndb.schema.fetch(req + where, values):
             route_id = record[-1]
             record = list(record[:-1])
-            #
-            # fetch metrics
-            metrics = tuple(view.ndb.schema.fetch('''
-                SELECT * FROM metrics WHERE f_route_id = %s
-            ''' % (plch, ), (route_id, )))
-            if metrics:
-                ret = {}
-                names = view.ndb.schema.compiled['metrics']['norm_names']
-                for k, v in zip(names, metrics[0]):
-                    if v is not None and \
-                            k not in ('target', 'route_id', 'tflags'):
-                        ret[k] = v
-                record.append(json.dumps(ret))
+            if route_id is not None:
+                #
+                # fetch metrics
+                metrics = tuple(
+                    view.ndb.schema.fetch(
+                        '''
+                    SELECT * FROM metrics WHERE f_route_id = %s
+                '''
+                        % (plch,),
+                        (route_id,),
+                    )
+                )
+                if metrics:
+                    ret = {}
+                    names = view.ndb.schema.compiled['metrics']['norm_names']
+                    for k, v in zip(names, metrics[0]):
+                        if v is not None and k not in (
+                            'target',
+                            'route_id',
+                            'tflags',
+                        ):
+                            ret[k] = v
+                    record.append(json.dumps(ret))
+                else:
+                    record.append(None)
+                #
+                # fetch encap
+                enc_mpls = tuple(
+                    view.ndb.schema.fetch(
+                        '''
+                    SELECT * FROM enc_mpls WHERE f_route_id = %s
+                '''
+                        % (plch,),
+                        (route_id,),
+                    )
+                )
+                if enc_mpls:
+                    record.append(enc_mpls[0][2])
+                else:
+                    record.append(None)
             else:
-                record.append(None)
-            #
-            # fetch encap
-            enc_mpls = tuple(view.ndb.schema.fetch('''
-                SELECT * FROM enc_mpls WHERE f_route_id = %s
-            ''' % (plch, ), (route_id, )))
-            if enc_mpls:
-                record.append(enc_mpls[0][2])
-            else:
-                record.append(None)
+                record.extend((None, None))
             yield record
 
-    @staticmethod
-    def spec_normalize(spec):
-        if isinstance(spec, dict):
-            ret = dict(spec)
-        elif isinstance(spec, basestring):
-            dst_spec = spec.split('/')
-            ret = {'dst': dst_spec[0]}
-            if len(dst_spec) == 2:
-                ret['dst_len'] = int(dst_spec[1])
-            elif len(dst_spec) > 2:
-                raise TypeError('invalid spec format')
-        else:
-            raise TypeError('invalid spec type')
-        if ret.get('dst') == 'default':
-            ret['dst'] = ''
-        elif ret.get('dst') in ('::', '::/0'):
-            ret['dst'] = ''
-            ret['family'] = AF_INET6
-        return ret
+    @classmethod
+    def spec_normalize(cls, processed, spec):
+        if isinstance(spec, basestring):
+            processed['dst'] = spec
+        return processed
+
+    @classmethod
+    def compare_record(self, left, right):
+        if isinstance(right, str):
+            return right == f'{left["dst"]}/{left["dst_len"]}'
 
     def _cmp_target(key, self, right):
-        right = [Target(x) for x in json.loads(right)]
+        right = [MPLSTarget(x) for x in json.loads(right)]
         return all([x[0] == x[1] for x in zip(self[key], right)])
 
     def _cmp_via(self, right):
@@ -495,31 +543,34 @@ class Route(RTNL_Object):
     def _cmp_encap(self, right):
         return all([x[0] == x[1] for x in zip(self.get('encap', []), right)])
 
-    fields_cmp = {'dst': partial(_cmp_target, 'dst'),
-                  'src': partial(_cmp_target, 'src'),
-                  'newdst': partial(_cmp_target, 'newdst'),
-                  'encap': _cmp_encap,
-                  'via': _cmp_via}
+    fields_cmp = {
+        'dst': partial(_cmp_target, 'dst'),
+        'src': partial(_cmp_target, 'src'),
+        'newdst': partial(_cmp_target, 'newdst'),
+        'encap': _cmp_encap,
+        'via': _cmp_via,
+    }
 
     def mark_tflags(self, mark):
-        plch = (self.schema.plch, ) * 4
-        self.schema.execute('''
+        plch = (self.schema.plch,) * 4
+        self.schema.execute(
+            '''
                             UPDATE interfaces SET
                                 f_tflags = %s
                             WHERE
                                 (f_index = %s OR f_index = %s)
                                 AND f_target = %s
-                            ''' % plch,
-                            (mark,
-                             self['iif'],
-                             self['oif'],
-                             self['target']))
+                            '''
+            % plch,
+            (mark, self['iif'], self['oif'], self['target']),
+        )
 
     def __init__(self, *argv, **kwarg):
         kwarg['iclass'] = rtmsg
         self.event_map = {rtmsg: "load_rtnlmsg"}
         dict.__setitem__(self, 'multipath', [])
         dict.__setitem__(self, 'metrics', {})
+        dict.__setitem__(self, 'deps', 0)
         super(Route, self).__init__(*argv, **kwarg)
 
     def complete_key(self, key):
@@ -536,9 +587,23 @@ class Route(RTNL_Object):
         if 'target' not in ret_key:
             ret_key['target'] = self.ndb.localhost
 
-        table = ret_key.get('table', ret_key.get('RTA_TABLE', 254))
-        if 'table' not in ret_key:
-            ret_key['table'] = table
+        ##
+        # previously here was a code that injected the default
+        # table == 254 into the key:
+        #
+        # table = ret_key.get('table', ret_key.get('RTA_TABLE', 254))
+        # if 'table' not in ret_key:
+        #     ret_key['table'] = table
+        #
+        # the issue with the code is that self.exists() didn't use
+        # it, thus it was possible to get self.exists() == True and
+        # at the same time loading from the DB resulted in an empty
+        # record
+        #
+        # probably more correct behaviour would be to raise KeyError
+        # if a route spec has no table defined, and the route is
+        # in another table than 254; but for now routes['ipaddr/mask']
+        # returns records even outside of the main table
 
         if isinstance(ret_key.get('dst_len'), basestring):
             ret_key['dst_len'] = int(ret_key['dst_len'])
@@ -561,7 +626,7 @@ class Route(RTNL_Object):
     @property
     def clean(self):
         clean = True
-        for s in (self['metrics'], ) + tuple(self['multipath']):
+        for s in (self['metrics'],) + tuple(self['multipath']):
             if hasattr(s, 'changed'):
                 clean &= len(s.changed) == 0
         return clean & super(Route, self).clean
@@ -575,26 +640,17 @@ class Route(RTNL_Object):
         if self['metrics']:
             req['metrics'] = self['metrics']
         if self.get('encap') and self.get('encap_type'):
-            req['encap'] = {'type': self['encap_type'],
-                            'labels': self['encap']}
+            req['encap'] = {
+                'type': self['encap_type'],
+                'labels': self['encap'],
+            }
         if self.get('gateway'):
             req['gateway'] = self['gateway']
         return req
 
     @check_auth('obj:modify')
     def __setitem__(self, key, value):
-        if key in ('dst', 'src') and \
-                isinstance(value, basestring) and \
-                '/' in value:
-            net, net_len = value.split('/')
-            if net in ('0', '0.0.0.0'):
-                net = ''
-            super(Route, self).__setitem__(key, net)
-            super(Route, self).__setitem__('%s_len' % key, int(net_len))
-        elif key == 'dst' and value == 'default':
-            super(Route, self).__setitem__('dst', '')
-            super(Route, self).__setitem__('dst_len', 0)
-        elif key == 'route_id':
+        if key == 'route_id':
             raise ValueError('route_id is read only')
         elif key == 'multipath':
             super(Route, self).__setitem__('multipath', [])
@@ -602,10 +658,9 @@ class Route(RTNL_Object):
                 mp = dict(mp)
                 if self.state == 'invalid':
                     mp['create'] = True
-                obj = NextHop(self,
-                              self.view,
-                              mp,
-                              auth_managers=self.auth_managers)
+                obj = NextHop(
+                    self, self.view, mp, auth_managers=self.auth_managers
+                )
                 obj.state.set(self.state.get())
                 self['multipath'].append(obj)
             if key in self.changed:
@@ -614,37 +669,24 @@ class Route(RTNL_Object):
             value = dict(value)
             if self.state == 'invalid':
                 value['create'] = True
-            obj = Metrics(self,
-                          self.view,
-                          value,
-                          auth_managers=self.auth_managers)
+            obj = Metrics(
+                self, self.view, value, auth_managers=self.auth_managers
+            )
             obj.state.set(self.state.get())
             super(Route, self).__setitem__('metrics', obj)
             if key in self.changed:
                 self.changed.remove(key)
-        elif key == 'encap':
-            if value.get('type') == 'mpls':
-                na = []
-                target = None
-                value = value.get('labels', [])
-                if isinstance(value, (dict, int)):
-                    value = [value, ]
-                for label in value:
-                    target = Target(label)
-                    target['bos'] = 0
-                    na.append(target)
-                target['bos'] = 1
-                super(Route, self).__setitem__('encap_type',
-                                               LWTUNNEL_ENCAP_MPLS)
-                super(Route, self).__setitem__('encap', na)
-        elif self.get('family', 0) == AF_MPLS and \
-                key in ('dst', 'src', 'newdst'):
+        elif self.get('family', 0) == AF_MPLS and key in (
+            'dst',
+            'src',
+            'newdst',
+        ):
             if isinstance(value, (dict, int)):
-                value = [value, ]
+                value = [value]
             na = []
             target = None
             for label in value:
-                target = Target(label)
+                target = MPLSTarget(label)
                 target['bos'] = 0
                 na.append(target)
             target['bos'] = 1
@@ -653,16 +695,18 @@ class Route(RTNL_Object):
             super(Route, self).__setitem__(key, value)
 
     @check_auth('obj:modify')
-    def apply(self, rollback=False):
-        if (self.get('table') == 255) and \
-                (self.get('family') == 10) and \
-                (self.get('proto') == 2):
+    def apply(self, rollback=False, req_filter=None, mode='apply'):
+        if (
+            (self.get('table') == 255)
+            and (self.get('family') == 10)
+            and (self.get('proto') == 2)
+        ):
             # skip automatic ipv6 routes with proto kernel
             return self
         else:
             if self.get('family', AF_INET) == AF_MPLS and not self.get('dst'):
-                dict.__setitem__(self, 'dst', [Target()])
-            return super(Route, self).apply(rollback)
+                dict.__setitem__(self, 'dst', [MPLSTarget()])
+            return super(Route, self).apply(rollback, req_filter, mode)
 
     def load_sql(self, *argv, **kwarg):
         super(Route, self).load_sql(*argv, **kwarg)
@@ -674,37 +718,63 @@ class Route(RTNL_Object):
                     if field == 'via':
                         na = json.loads(value)
                     else:
-                        na = [Target(x) for x in json.loads(value)]
+                        na = [MPLSTarget(x) for x in json.loads(value)]
                     dict.__setitem__(self, field, na)
-
-        if self.get('route_id') is not None:
-            enc = (self
-                   .schema
-                   .fetch('SELECT * FROM enc_mpls WHERE f_route_id = %s' %
-                          (self.schema.plch, ), (self['route_id'], )))
-            enc = tuple(enc)
-            if enc:
-                na = [Target(x) for x in json.loads(enc[0][2])]
-                self.load_value('encap', na)
-
+        #
+        # fetch encap deps
+        if self['deps'] & F_RTA_ENCAP:
+            for _ in range(5):
+                enc = tuple(
+                    self.schema.fetch(
+                        'SELECT * FROM enc_mpls WHERE f_route_id = %s'
+                        % (self.schema.plch,),
+                        (self['route_id'],),
+                    )
+                )
+                if enc:
+                    na = [MPLSTarget(x) for x in json.loads(enc[0][2])]
+                    self.load_value('encap', na)
+                    break
+                time.sleep(0.1)
+            else:
+                self.log.error('no encap loaded for %s' % (self['route_id'],))
+        #
+        #
         if not self.load_event.is_set():
             return
-
+        #
+        # fetch metrics
+        if self['deps'] & F_RTA_METRICS:
+            for _ in range(5):
+                metrics = tuple(
+                    self.schema.fetch(
+                        'SELECT * FROM metrics WHERE f_route_id = %s'
+                        % (self.schema.plch,),
+                        (self['route_id'],),
+                    )
+                )
+                if metrics:
+                    self['metrics'] = Metrics(
+                        self,
+                        self.view,
+                        {'route_id': self['route_id']},
+                        auth_managers=self.auth_managers,
+                    )
+                    break
+                time.sleep(0.1)
+            else:
+                self.log.error(
+                    'no metrics loaded for %s' % (self['route_id'],)
+                )
+        #
+        # fetch multipath
+        #
+        # FIXME: use self['deps']
         if 'nh_id' not in self and self.get('route_id') is not None:
-            nhs = (self
-                   .schema
-                   .fetch('SELECT * FROM nh WHERE f_route_id = %s' %
-                          (self.schema.plch, ), (self['route_id'], )))
-            metrics = (self
-                       .schema
-                       .fetch('SELECT * FROM metrics WHERE f_route_id = %s' %
-                              (self.schema.plch, ), (self['route_id'], )))
-
-            if tuple(metrics):
-                self['metrics'] = Metrics(self,
-                                          self.view,
-                                          {'route_id': self['route_id']},
-                                          auth_managers=self.auth_managers)
+            nhs = self.schema.fetch(
+                'SELECT * FROM nh WHERE f_route_id = %s' % (self.schema.plch,),
+                (self['route_id'],),
+            )
 
             flush = False
             idx = 0
@@ -728,22 +798,25 @@ class Route(RTNL_Object):
                 idx += 1
 
             for nexthop in nhs:
-                key = {'route_id': self['route_id'],
-                       'nh_id': nexthop[-1]}
-                (self['multipath']
-                 .append(NextHop(self,
-                                 self.view,
-                                 key,
-                                 auth_managers=self.auth_managers)))
+                key = {'route_id': self['route_id'], 'nh_id': nexthop[-1]}
+                (
+                    self['multipath'].append(
+                        NextHop(
+                            self,
+                            self.view,
+                            key,
+                            auth_managers=self.auth_managers,
+                        )
+                    )
+                )
 
 
 class RouteSub(object):
+    def apply(self, rollback=False, req_filter=None, mode='apply'):
+        return self.route.apply(rollback, req_filter, mode)
 
-    def apply(self, *argv, **kwarg):
-        return self.route.apply(*argv, **kwarg)
-
-    def commit(self, *argv, **kwarg):
-        return self.route.commit(*argv, **kwarg)
+    def commit(self):
+        return self.route.commit()
 
 
 class NextHop(RouteSub, RTNL_Object):
@@ -753,18 +826,18 @@ class NextHop(RouteSub, RTNL_Object):
     hidden_fields = ('route_id', 'target')
 
     def mark_tflags(self, mark):
-        plch = (self.schema.plch, ) * 4
-        self.schema.execute('''
+        plch = (self.schema.plch,) * 4
+        self.schema.execute(
+            '''
                             UPDATE interfaces SET
                                 f_tflags = %s
                             WHERE
                                 (f_index = %s OR f_index = %s)
                                 AND f_target = %s
-                            ''' % plch,
-                            (mark,
-                             self.route['iif'],
-                             self.route['oif'],
-                             self.route['target']))
+                            '''
+            % plch,
+            (mark, self.route['iif'], self.route['oif'], self.route['target']),
+        )
 
     def __init__(self, route, *argv, **kwarg):
         self.route = route
@@ -780,18 +853,18 @@ class Metrics(RouteSub, RTNL_Object):
     hidden_fields = ('route_id', 'target')
 
     def mark_tflags(self, mark):
-        plch = (self.schema.plch, ) * 4
-        self.schema.execute('''
+        plch = (self.schema.plch,) * 4
+        self.schema.execute(
+            '''
                             UPDATE interfaces SET
                                 f_tflags = %s
                             WHERE
                                 (f_index = %s OR f_index = %s)
                                 AND f_target = %s
-                            ''' % plch,
-                            (mark,
-                             self.route['iif'],
-                             self.route['oif'],
-                             self.route['target']))
+                            '''
+            % plch,
+            (mark, self.route['iif'], self.route['oif'], self.route['target']),
+        )
 
     def __init__(self, route, *argv, **kwarg):
         self.route = route
