@@ -3,16 +3,15 @@
 Usage::
 
     # Imports
-    from pyroute2 import IPDB, WireGuard
+    from pyroute2 import NDB, WireGuard
 
     IFNAME = 'wg1'
 
     # Create a WireGuard interface
-    with IPDB() as ip:
-        wg1 = ip.create(kind='wireguard', ifname=IFNAME)
-        wg1.add_ip('10.0.0.1/24')
-        wg1.up()
-        wg1.commit()
+    with NDB() as ndb:
+        with ndb.interfaces.create(kind='wireguard', ifname=IFNAME) as link:
+            link.add_ip('10.0.0.1/24')
+            link.set(state='up')
 
     # Create WireGuard object
     wg = WireGuard()
@@ -64,23 +63,24 @@ NOTES:
 '''
 
 
-from base64 import b64encode, b64decode
-from binascii import a2b_hex
 import errno
 import logging
-from socket import inet_ntoa, inet_aton, inet_pton, AF_INET, AF_INET6
-from struct import pack, unpack
+import struct
+from base64 import b64decode, b64encode
+from binascii import a2b_hex
+from socket import AF_INET, AF_INET6, inet_ntop, inet_pton
 from time import ctime
 
-from pyroute2.netlink import genlmsg
-from pyroute2.netlink import nla
-from pyroute2.netlink import NLM_F_ACK
-from pyroute2.netlink import NLM_F_DUMP
-from pyroute2.netlink import NLA_F_NESTED
-from pyroute2.netlink import NLM_F_REQUEST
+from pyroute2.netlink import (
+    NLA_F_NESTED,
+    NLM_F_ACK,
+    NLM_F_DUMP,
+    NLM_F_REQUEST,
+    genlmsg,
+    nla,
+)
 from pyroute2.netlink.generic import GenericNetlinkSocket
 
-
 # Defines from uapi/wireguard.h
 WG_GENL_NAME = "wireguard"
 WG_GENL_VERSION = 1
@@ -123,35 +123,38 @@ WG_MAX_ALLOWEDIPS = 1000
 class wgmsg(genlmsg):
     prefix = 'WGDEVICE_A_'
 
-    nla_map = (('WGDEVICE_A_UNSPEC', 'none'),
-               ('WGDEVICE_A_IFINDEX', 'uint32'),
-               ('WGDEVICE_A_IFNAME', 'asciiz'),
-               ('WGDEVICE_A_PRIVATE_KEY', 'parse_wg_key'),
-               ('WGDEVICE_A_PUBLIC_KEY', 'parse_wg_key'),
-               ('WGDEVICE_A_FLAGS', 'uint32'),
-               ('WGDEVICE_A_LISTEN_PORT', 'uint16'),
-               ('WGDEVICE_A_FWMARK', 'uint32'),
-               ('WGDEVICE_A_PEERS', '*wgdevice_peer'))
+    nla_map = (
+        ('WGDEVICE_A_UNSPEC', 'none'),
+        ('WGDEVICE_A_IFINDEX', 'uint32'),
+        ('WGDEVICE_A_IFNAME', 'asciiz'),
+        ('WGDEVICE_A_PRIVATE_KEY', 'parse_wg_key'),
+        ('WGDEVICE_A_PUBLIC_KEY', 'parse_wg_key'),
+        ('WGDEVICE_A_FLAGS', 'uint32'),
+        ('WGDEVICE_A_LISTEN_PORT', 'uint16'),
+        ('WGDEVICE_A_FWMARK', 'uint32'),
+        ('WGDEVICE_A_PEERS', '*wgdevice_peer'),
+    )
 
     class wgdevice_peer(nla):
         prefix = 'WGPEER_A_'
 
         nla_flags = NLA_F_NESTED
-        nla_map = (('WGPEER_A_UNSPEC', 'none'),
-                   ('WGPEER_A_PUBLIC_KEY', 'parse_peer_key'),
-                   ('WGPEER_A_PRESHARED_KEY', 'parse_peer_key'),
-                   ('WGPEER_A_FLAGS', 'uint32'),
-                   ('WGPEER_A_ENDPOINT', 'parse_endpoint'),
-                   ('WGPEER_A_PERSISTENT_KEEPALIVE_INTERVAL', 'uint16'),
-                   ('WGPEER_A_LAST_HANDSHAKE_TIME',
-                    'parse_handshake_time'),
-                   ('WGPEER_A_RX_BYTES', 'uint64'),
-                   ('WGPEER_A_TX_BYTES', 'uint64'),
-                   ('WGPEER_A_ALLOWEDIPS', '*wgpeer_allowedip'),
-                   ('WGPEER_A_PROTOCOL_VERSION', 'uint32'))
+        nla_map = (
+            ('WGPEER_A_UNSPEC', 'none'),
+            ('WGPEER_A_PUBLIC_KEY', 'parse_peer_key'),
+            ('WGPEER_A_PRESHARED_KEY', 'parse_peer_key'),
+            ('WGPEER_A_FLAGS', 'uint32'),
+            ('WGPEER_A_ENDPOINT', 'parse_endpoint'),
+            ('WGPEER_A_PERSISTENT_KEEPALIVE_INTERVAL', 'uint16'),
+            ('WGPEER_A_LAST_HANDSHAKE_TIME', 'parse_handshake_time'),
+            ('WGPEER_A_RX_BYTES', 'uint64'),
+            ('WGPEER_A_TX_BYTES', 'uint64'),
+            ('WGPEER_A_ALLOWEDIPS', '*wgpeer_allowedip'),
+            ('WGPEER_A_PROTOCOL_VERSION', 'uint32'),
+        )
 
         class parse_peer_key(nla):
-            fields = (('key', '32s'), )
+            fields = (('key', '32s'),)
 
             def decode(self):
                 nla.decode(self)
@@ -161,37 +164,60 @@ class wgmsg(genlmsg):
                 self['key'] = b64decode(self['value'])
                 nla.encode(self)
 
-        class parse_endpoint(nla):
-            fields = (('family', 'H'),
-                      ('port', '>H'),
-                      ('addr4', '>I'),
-                      ('addr6', 's'))
+        @staticmethod
+        def parse_endpoint(nla, *argv, **kwarg):
+            family = AF_INET
+            if 'data' in kwarg:
+                # decoding, fetch the famliy from the NLA data
+                data = kwarg['data']
+                offset = kwarg['offset']
+                family = struct.unpack('H', data[offset + 4 : offset + 6])[0]
+            elif kwarg['value']['addr'].find(':') > -1:
+                # encoding, setup family from the addr format
+                family = AF_INET6
+
+            if family == AF_INET:
+                return nla.endpoint_ipv4
+            else:
+                return nla.endpoint_ipv6
+
+        class endpoint_ipv4(nla):
+            fields = (
+                ('family', 'H'),
+                ('port', '>H'),
+                ('addr', '4s'),
+                ('__pad', '8x'),
+            )
 
             def decode(self):
                 nla.decode(self)
-                if self['family'] == AF_INET:
-                    self['addr'] = inet_ntoa(pack('>I', self['addr4']))
-                else:
-                    self['addr'] = inet_ntoa(AF_INET6, self['addr6'])
-                del self['addr4']
-                del self['addr6']
+                self['addr'] = inet_ntop(AF_INET, self['addr'])
 
             def encode(self):
-                if self['addr'].find(":") > -1:
-                    self['family'] = AF_INET6
-                    self['addr4'] = 0  # Set to NULL
-                    self['addr6'] = inet_pton(AF_INET6, self['addr'])
-                else:
-                    self['family'] = AF_INET
-                    self['addr4'] = unpack('>I',
-                                           inet_aton(self['addr']))[0]
-                    self['addr6'] = b'\x00\x00\x00\x00\x00\x00\x00\x00'
-                self['port'] = int(self['port'])
+                self['family'] = AF_INET
+                self['addr'] = inet_pton(AF_INET, self['addr'])
+                nla.encode(self)
+
+        class endpoint_ipv6(nla):
+            fields = (
+                ('family', 'H'),
+                ('port', '>H'),
+                ('flowinfo', '>I'),
+                ('addr', '16s'),
+                ('scope_id', '>I'),
+            )
+
+            def decode(self):
+                nla.decode(self)
+                self['addr'] = inet_ntop(AF_INET6, self['addr'])
+
+            def encode(self):
+                self['family'] = AF_INET6
+                self['addr'] = inet_pton(AF_INET6, self['addr'])
                 nla.encode(self)
 
         class parse_handshake_time(nla):
-            fields = (('tv_sec', 'Q'),
-                      ('tv_nsec', 'Q'))
+            fields = (('tv_sec', 'Q'), ('tv_nsec', 'Q'))
 
             def decode(self):
                 nla.decode(self)
@@ -201,26 +227,30 @@ class wgmsg(genlmsg):
             prefix = 'WGALLOWEDIP_A_'
 
             nla_flags = NLA_F_NESTED
-            nla_map = (('WGALLOWEDIP_A_UNSPEC', 'none'),
-                       ('WGALLOWEDIP_A_FAMILY', 'uint16'),
-                       ('WGALLOWEDIP_A_IPADDR', 'hex'),
-                       ('WGALLOWEDIP_A_CIDR_MASK', 'uint8'))
+            nla_map = (
+                ('WGALLOWEDIP_A_UNSPEC', 'none'),
+                ('WGALLOWEDIP_A_FAMILY', 'uint16'),
+                ('WGALLOWEDIP_A_IPADDR', 'hex'),
+                ('WGALLOWEDIP_A_CIDR_MASK', 'uint8'),
+            )
 
             def decode(self):
                 nla.decode(self)
-                if self.get_attr('WGALLOWEDIP_A_FAMILY') == AF_INET:
-                    pre = (self
-                           .get_attr('WGALLOWEDIP_A_IPADDR')
-                           .replace(':', ''))
-                    self['addr'] = inet_ntoa(a2b_hex(pre))
-                else:
-                    self['addr'] = (self
-                                    .get_attr('WGALLOWEDIP_A_IPADDR'))
-                wgaddr = self.get_attr('WGALLOWEDIP_A_CIDR_MASK')
-                self['addr'] = '{0}/{1}'.format(self['addr'], wgaddr)
+                family = self.get_attr('WGALLOWEDIP_A_FAMILY')
+                if family is None:
+                    # Prevent when decode() is called without attrs because all
+                    # datas transfered to 'value' entry.
+                    #  {'attrs': [], 'value': [{'attrs' ...
+                    return
+                ipaddr = self.get_attr('WGALLOWEDIP_A_IPADDR')
+                cidr = self.get_attr('WGALLOWEDIP_A_CIDR_MASK')
+                self['addr'] = '{ipaddr}/{cidr}'.format(
+                    ipaddr=inet_ntop(family, a2b_hex(ipaddr.replace(':', ''))),
+                    cidr=cidr,
+                )
 
     class parse_wg_key(nla):
-        fields = (('key', '32s'), )
+        fields = (('key', '32s'),)
 
         def decode(self):
             nla.decode(self)
@@ -232,26 +262,26 @@ class wgmsg(genlmsg):
 
 
 class WireGuard(GenericNetlinkSocket):
-
     def __init__(self):
         GenericNetlinkSocket.__init__(self)
         self.bind(WG_GENL_NAME, wgmsg)
 
-    def info(self,
-             interface):
+    def info(self, interface):
         msg = wgmsg()
         msg['cmd'] = WG_CMD_GET_DEVICE
         msg['attrs'].append(['WGDEVICE_A_IFNAME', interface])
-        return self.nlm_request(msg,
-                                msg_type=self.prid,
-                                msg_flags=NLM_F_REQUEST | NLM_F_DUMP)
+        return self.nlm_request(
+            msg, msg_type=self.prid, msg_flags=NLM_F_REQUEST | NLM_F_DUMP
+        )
 
-    def set(self,
-            interface,
-            listen_port=None,
-            fwmark=None,
-            private_key=None,
-            peer=None):
+    def set(
+        self,
+        interface,
+        listen_port=None,
+        fwmark=None,
+        private_key=None,
+        peer=None,
+    ):
         msg = wgmsg()
         msg['attrs'].append(['WGDEVICE_A_IFNAME', interface])
 
@@ -280,8 +310,9 @@ class WireGuard(GenericNetlinkSocket):
         err = msg['header'].get('error', None)
         if err is not None:
             if hasattr(err, 'code') and err.code == errno.ENOENT:
-                logging.error('Generic netlink protocol %s not found'
-                              % self.prid)
+                logging.error(
+                    'Generic netlink protocol %s not found' % self.prid
+                )
                 logging.error('Please check if the protocol module is loaded')
             raise err
         return msg
@@ -312,9 +343,15 @@ class WireGuard(GenericNetlinkSocket):
 
         # Set Endpoint
         if 'endpoint_addr' in peer and 'endpoint_port' in peer:
-            attrs.append(['WGPEER_A_ENDPOINT',
-                          {'addr': peer['endpoint_addr'],
-                           'port': peer['endpoint_port']}])
+            attrs.append(
+                [
+                    'WGPEER_A_ENDPOINT',
+                    {
+                        'addr': peer['endpoint_addr'],
+                        'port': peer['endpoint_port'],
+                    },
+                ]
+            )
 
         # Set Preshared key
         if 'preshared_key' in peer:
@@ -325,8 +362,7 @@ class WireGuard(GenericNetlinkSocket):
         # Set Persistent Keepalive time
         if 'persistent_keepalive' in peer:
             keepalive = peer['persistent_keepalive']
-            attrs.append(['WGPEER_A_PERSISTENT_KEEPALIVE_INTERVAL',
-                          keepalive])
+            attrs.append(['WGPEER_A_PERSISTENT_KEEPALIVE_INTERVAL', keepalive])
 
         # Set Peer flags
         attrs.append(['WGPEER_A_FLAGS', WGPEER_F_UPDATE_ONLY])
@@ -349,13 +385,12 @@ class WireGuard(GenericNetlinkSocket):
                 raise ValueError('No CIDR set in allowed ip #{}'.format(index))
 
             addr, mask = ip.split('/')
-            if addr.find(":") > -1:
-                allowed_ip.append(['WGALLOWEDIP_A_FAMILY', AF_INET6])
-                allowed_ip.append(['WGALLOWEDIP_A_IPADDR', inet_pton(AF_INET6,
-                                                                     addr)])
-            else:
-                allowed_ip.append(['WGALLOWEDIP_A_FAMILY', AF_INET])
-                allowed_ip.append(['WGALLOWEDIP_A_IPADDR', inet_aton(addr)])
+
+            family = AF_INET if addr.find(":") == -1 else AF_INET6
+            allowed_ip.append(['WGALLOWEDIP_A_FAMILY', family])
+            allowed_ip.append(
+                ['WGALLOWEDIP_A_IPADDR', inet_pton(family, addr)]
+            )
             allowed_ip.append(['WGALLOWEDIP_A_CIDR_MASK', int(mask)])
 
         return ret
